@@ -11,6 +11,7 @@ from pathlib import Path
 from wsgiref.util import FileWrapper
 
 import pandas as pd
+import requests
 from core import utils
 from core.feature_flags import all_flags, get_feature_file_path
 from core.label_config import generate_time_series_json
@@ -29,6 +30,8 @@ from django.http import (
 from django.shortcuts import redirect, reverse
 from django.template import loader
 from django.utils._os import safe_join
+from django.views.decorators.csrf import csrf_exempt
+from django.views.decorators.http import require_http_methods
 from drf_yasg.utils import swagger_auto_schema
 from io_storages.localfiles.models import LocalFilesImportStorage
 from ranged_fileresponse import RangedFileResponse
@@ -69,7 +72,7 @@ def version_page(request):
     # html / json response
     if request.path == '/version/':
         # other settings from backend
-        if request.user.is_superuser:
+        if not getattr(settings, 'CLOUD_INSTANCE', False) and request.user.is_superuser:
             result['settings'] = {
                 key: str(getattr(settings, key))
                 for key in dir(settings)
@@ -169,6 +172,33 @@ def samples_paragraphs(request):
     return HttpResponse(json.dumps(result), content_type='application/json')
 
 
+def heidi_tips(request):
+    """Fetch live tips from github raw liveContent.json to avoid caching and client side CORS issues"""
+    url = 'https://raw.githubusercontent.com/HumanSignal/label-studio/refs/heads/develop/web/apps/labelstudio/src/components/HeidiTips/liveContent.json'
+
+    response = None
+    try:
+        response = requests.get(
+            url,
+            headers={'Cache-Control': 'no-cache', 'Content-Type': 'application/json', 'Accept': 'application/json'},
+            timeout=5,
+        )
+        # Raise an exception for bad status codes to avoid caching
+        response.raise_for_status()
+    # Catch all exceptions and return either the status code if there was a response, or default to 404 if there are network issues
+    # This is done this way to catch thrown exceptions from the request itself which will occur for air-gapped environments
+    except Exception:
+        # Any other HTTP error will return the error code, and other errors like connection/timeout errors will be a 404
+        content = {}
+        status_code = 404
+        if response is not None:
+            content['detail'] = response.reason
+            status_code = response.status_code
+        return HttpResponse(json.dumps(content), content_type='application/json', status=status_code)
+
+    return HttpResponse(response.content, content_type='application/json')
+
+
 @swagger_auto_schema(methods=['GET'], auto_schema=None)
 @api_view(['GET'])
 @permission_classes([IsAuthenticated])
@@ -244,3 +274,10 @@ def feature_flags(request):
     }
 
     return HttpResponse('<pre>' + json.dumps(flags, indent=4) + '</pre>', status=200)
+
+
+@csrf_exempt
+@require_http_methods(['POST', 'GET'])
+def collect_metrics(request):
+    """Lightweight endpoint to collect usage metrics from the frontend only when COLLECT_ANALYTICS is enabled"""
+    return HttpResponse(status=204)
