@@ -137,6 +137,9 @@ class ExportMixin:
                 options['context']['interpolate_key_frames'] = serialization_options['interpolate_key_frames']
             if serialization_options.get('include_annotation_history') is False:
                 options['omit'] = ['annotations.history']
+            # download resources
+            if serialization_options.get('download_resources') is True:
+                options['download_resources'] = True
         return options
 
     def get_task_queryset(self, ids, annotation_filter_options):
@@ -303,7 +306,7 @@ class ExportMixin:
                 serialization_options=serialization_options,
             )
 
-    def convert_file(self, to_format):
+    def convert_file(self, to_format, download_resources=False, hostname=None):
         with get_temp_dir() as tmp_dir:
             OUT = 'out'
             out_dir = pathlib.Path(tmp_dir) / OUT
@@ -313,7 +316,10 @@ class ExportMixin:
                 config=self.project.get_parsed_config(),
                 project_dir=None,
                 upload_dir=out_dir,
-                download_resources=False,
+                download_resources=download_resources,
+                # for downloading resource we need access to the API
+                access_token=self.project.organization.created_by.auth_token.key,
+                hostname=hostname,
             )
             input_name = pathlib.Path(self.file.name).name
             input_file_path = pathlib.Path(tmp_dir) / input_name
