@@ -11,7 +11,6 @@ from core.utils.db import fast_first
 from data_manager.prepare_params import ConjunctionEnum
 from django.conf import settings
 from django.contrib.postgres.aggregates import ArrayAgg
-from django.contrib.postgres.fields.jsonb import KeyTextTransform
 from django.db import models
 from django.db.models import (
     Aggregate,
@@ -28,6 +27,7 @@ from django.db.models import (
     Value,
     When,
 )
+from django.db.models.fields.json import KeyTextTransform
 from django.db.models.functions import Cast, Coalesce, Concat
 from pydantic import BaseModel
 
@@ -149,9 +149,7 @@ def apply_ordering(queryset, ordering, project, request, view_data=None):
             and view_data['columnsDisplayType'][unsigned_field_name] == 'Number'
         ):
             numeric_ordering = True
-        field_name, ascending = preprocess_field_name(
-            raw_field_name, only_undefined_field=project.only_undefined_field
-        )
+        field_name, ascending = preprocess_field_name(raw_field_name, project=project)
 
         if field_name.startswith('data__'):
             # annotate task with data field for float/int/bool ordering support
@@ -275,7 +273,7 @@ def apply_filters(queryset, filters, project, request):
 
         # django orm loop expression attached to column name
         preprocess_field_name = load_func(settings.PREPROCESS_FIELD_NAME)
-        field_name, _ = preprocess_field_name(_filter.filter, project.only_undefined_field)
+        field_name, _ = preprocess_field_name(_filter.filter, project)
 
         # filter pre-processing, value type conversion, etc..
         preprocess_filter = load_func(settings.DATA_MANAGER_PREPROCESS_FILTER)
@@ -601,7 +599,7 @@ def annotate_annotations_results(queryset):
             )
         )
     else:
-        return queryset.annotate(annotations_results=ArrayAgg('annotations__result', distinct=True))
+        return queryset.annotate(annotations_results=ArrayAgg('annotations__result', distinct=True, default=Value([])))
 
 
 def annotate_predictions_results(queryset):
@@ -612,7 +610,7 @@ def annotate_predictions_results(queryset):
             )
         )
     else:
-        return queryset.annotate(predictions_results=ArrayAgg('predictions__result', distinct=True))
+        return queryset.annotate(predictions_results=ArrayAgg('predictions__result', distinct=True, default=Value([])))
 
 
 def annotate_annotators(queryset):
@@ -621,7 +619,7 @@ def annotate_annotators(queryset):
             annotators=Coalesce(GroupConcat('annotations__completed_by'), Value(''), output_field=models.CharField())
         )
     else:
-        return queryset.annotate(annotators=ArrayAgg('annotations__completed_by', distinct=True))
+        return queryset.annotate(annotators=ArrayAgg('annotations__completed_by', distinct=True, default=Value([])))
 
 
 def annotate_predictions_score(queryset):
@@ -655,7 +653,7 @@ def annotate_annotations_ids(queryset):
     if settings.DJANGO_DB == settings.DJANGO_DB_SQLITE:
         return queryset.annotate(annotations_ids=GroupConcat('annotations__id', output_field=models.CharField()))
     else:
-        return queryset.annotate(annotations_ids=ArrayAgg('annotations__id'))
+        return queryset.annotate(annotations_ids=ArrayAgg('annotations__id', default=Value([])))
 
 
 def annotate_predictions_model_versions(queryset):
@@ -664,7 +662,7 @@ def annotate_predictions_model_versions(queryset):
             predictions_model_versions=GroupConcat('predictions__model_version', output_field=models.CharField())
         )
     else:
-        return queryset.annotate(predictions_model_versions=ArrayAgg('predictions__model_version'))
+        return queryset.annotate(predictions_model_versions=ArrayAgg('predictions__model_version', default=Value([])))
 
 
 def annotate_avg_lead_time(queryset):
