@@ -2,6 +2,7 @@
 """
 import json
 import logging
+from typing import Union
 
 from core.redis import start_job_async_or_sync
 from django.conf import settings
@@ -17,6 +18,7 @@ from io_storages.base_models import (
     ProjectStorageMixin,
 )
 from io_storages.gcs.utils import GCS
+from io_storages.utils import storage_can_resolve_bucket_url
 from tasks.models import Annotation
 
 logger = logging.getLogger(__name__)
@@ -93,6 +95,9 @@ class GCSImportStorageBase(GCSStorageMixin, ImportStorage):
             presign_ttl=self.presign_ttl,
         )
 
+    def can_resolve_url(self, url: Union[str, None]) -> bool:
+        return storage_can_resolve_bucket_url(self, url)
+
     def scan_and_create_links(self):
         return self._scan_and_create_links(GCSImportStorageLink)
 
