@@ -159,11 +159,7 @@ class MLApi(BaseHTTPAPI):
                 response = self.get(url=url, *args, **kwargs)
             response.raise_for_status()
         except requests.exceptions.RequestException as e:
-            # Extending error details in case of failed request
-            if flag_set('fix_back_dev_3351_ml_validation_error_extension_short', AnonymousUser):
-                error_string = str(e) + (' ' + str(response.text) if response else '')
-            else:
-                error_string = str(e)
+            error_string = str(e)
             status_code = response.status_code if response is not None else 0
             return MLApiResult(url, request, {'error': error_string}, headers, 'error', status_code=status_code)
         status_code = response.status_code
