@@ -1,17 +1,17 @@
 """This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
 """
 import logging
+from typing import Dict, List
 
-from core.utils.common import conditional_atomic, db_is_not_sqlite, load_func, safe_float
+from core.utils.common import conditional_atomic, db_is_not_sqlite, load_func
 from django.conf import settings
-from django.db import models
-from django.db.models import Count, Q
-from django.db.models.signals import post_save
+from django.db import models, transaction
+from django.db.models import Count, JSONField, Q
+from django.db.models.signals import post_save, pre_delete
 from django.dispatch import receiver
 from django.utils.translation import gettext_lazy as _
-from ml.api_connector import MLApi
+from ml.api_connector import PREDICT_URL, TIMEOUT_PREDICT, MLApi
 from projects.models import Project
-from tasks.models import Prediction
 from tasks.serializers import PredictionSerializer, TaskSimpleSerializer
 from webhooks.serializers import Webhook, WebhookSerializer
 
@@ -30,6 +30,11 @@ class MLBackendState(models.TextChoices):
     PREDICTING = 'PR', _('Predicting')
 
 
+class MLBackendAuth(models.TextChoices):
+    NONE = 'NONE', _('None')
+    BASIC_AUTH = 'BASIC_AUTH', _('Basic Auth')
+
+
 class MLBackend(models.Model):
     """ """
 
@@ -60,6 +65,29 @@ class MLBackend(models.Model):
         default='default',
         help_text='Name of the machine learning backend',
     )
+
+    auth_method = models.CharField(
+        max_length=255,
+        choices=MLBackendAuth.choices,
+        default=MLBackendAuth.NONE,
+    )
+
+    basic_auth_user = models.TextField(
+        _('basic auth user'),
+        blank=True,
+        null=True,
+        default='',
+        help_text='HTTP Basic Auth user',
+    )
+
+    basic_auth_pass = models.TextField(
+        _('basic auth password'),
+        blank=True,
+        null=True,
+        default='',
+        help_text='HTTP Basic Auth password',
+    )
+
     description = models.TextField(
         _('description'),
         blank=True,
@@ -67,6 +95,13 @@ class MLBackend(models.Model):
         default='',
         help_text='Description for the machine learning backend',
     )
+
+    extra_params = JSONField(
+        _('extra params'),
+        null=True,
+        help_text='Any extra parameters passed to the ML Backend during the setup',
+    )
+
     model_version = models.TextField(
         _('model version'),
         blank=True,
@@ -96,36 +131,76 @@ class MLBackend(models.Model):
     def __str__(self):
         return f'{self.title} (id={self.id}, url={self.url})'
 
+    def __init__(self, *args, **kwargs):
+        super(MLBackend, self).__init__(*args, **kwargs)
+        self.__original_title = self.title
+
+    def save(self, *args, **kwargs):
+        """
+        Overrides the save() method to update the associated project's model_version field.
+        If the title of the model instance is changed and the model_version
+        of the related project is currently the same as the original title,
+        the project's model_version is updated to the new title.
+        """
+        p = self.project
+
+        if self.title != self.__original_title and p.model_version == self.__original_title:
+            with transaction.atomic():
+                p.model_version = self.title
+                p.save(update_fields=['model_version'])
+                super().save(*args, **kwargs)
+                # reset original field to current field after save
+                self.__original_title = self.title
+        else:
+            super().save(*args, **kwargs)
+
     @staticmethod
-    def healthcheck_(url):
-        return MLApi(url=url).health()
+    def healthcheck_(url, auth_method=None, **kwargs):
+        return MLApi(url=url, auth_method=auth_method, **kwargs).health()
 
     def has_permission(self, user):
         user.project = self.project  # link for activity log
         return self.project.has_permission(user)
 
     @staticmethod
-    def setup_(url, project, model_version=None):
-        api = MLApi(url=url)
+    def setup_(url, project, auth_method=None, **kwargs):
+        api = MLApi(url=url, auth_method=auth_method, **kwargs)
+
         if not isinstance(project, Project):
             project = Project.objects.get(pk=project)
-        return api.setup(project, model_version=model_version)
+        return api.setup(project, **kwargs)
 
     def healthcheck(self):
-        return self.healthcheck_(self.url)
+        return self.healthcheck_(
+            self.url, self.auth_method, basic_auth_user=self.basic_auth_user, basic_auth_pass=self.basic_auth_pass
+        )
 
     def setup(self):
-        return self.setup_(self.url, self.project, None if self.auto_update else self.model_version)
+        return self.setup_(
+            self.url,
+            self.project,
+            self.auth_method,
+            extra_params=self.extra_params,
+            basic_auth_user=self.basic_auth_user,
+            basic_auth_pass=self.basic_auth_pass,
+        )
 
     @property
     def api(self):
-        return MLApi(url=self.url, timeout=self.timeout)
+        return MLApi(
+            url=self.url,
+            timeout=self.timeout,
+            auth_method=self.auth_method,
+            basic_auth_user=self.basic_auth_user,
+            basic_auth_pass=self.basic_auth_pass,
+        )
 
     @property
     def not_ready(self):
         return self.state in (MLBackendState.DISCONNECTED, MLBackendState.ERROR)
 
     def update_state(self):
+        model_version = None
         if self.healthcheck().is_error:
             self.state = MLBackendState.DISCONNECTED
         else:
@@ -143,6 +218,7 @@ class MLBackend(models.Model):
                     self.model_version = model_version
                 self.error_message = None
         self.save()
+        return model_version
 
     def train(self):
         train_response = self.api.train(self.project)
@@ -156,8 +232,115 @@ class MLBackend(models.Model):
                 MLBackendTrainJob.objects.create(job_id=current_train_job, ml_backend=self)
         self.save()
 
+    def _predict(self, task):
+        """This is low level prediction method that is used for debugging"""
+        ml_api = self.api
+        task_ser = TaskSimpleSerializer(task).data
+
+        request_params = ml_api._prep_prediction_req([task_ser], self.project)
+        ml_api_result = ml_api._request(PREDICT_URL, request_params, verbose=False, timeout=TIMEOUT_PREDICT)
+
+        if ml_api_result.is_error:
+            logger.info(f'Prediction not created for project {self}: {ml_api_result.error_message}')
+            return
+
+        results = ml_api_result.response.get('results', None)
+
+        return {
+            'status': 200,
+            'data': {
+                'status': ml_api_result.status_code,
+                'error_message': ml_api_result.error_message,
+                'url': ml_api._get_url(PREDICT_URL),
+                'task': task_ser,
+                'request': request_params,
+                'response': results,
+            },
+        }
+
+    def _get_predictions_from_ml_backend_one_by_one(
+        self, serialized_tasks: List[Dict], current_responses: List[Dict]
+    ) -> List[Dict]:
+        """
+        This is helper method to get predictions from ML backend one by one
+        in case when tasks length doesn't match responses length
+        Note: don't use this function outside of this class
+        """
+
+        if len(current_responses) == 1:
+            # In case ML backend doesn't support batch of tasks, do it one by one
+            # TODO: remove this block after all ML backends will support batch processing
+            logger.warning(
+                f"'ML backend '{self.title}' doesn't support batch processing of tasks, "
+                f'switched to one-by-one task retrieval'
+            )
+            predictions = []
+            for serialized_task in serialized_tasks:
+                # get predictions per task
+                predictions.extend(self._get_predictions_from_ml_backend([serialized_task]))
+
+            return predictions
+        else:
+            # complete failure - likely ML backend skipped some tasks, we can't match them
+            logger.error(
+                f'Number of tasks and responses are not equal: '
+                f'{len(serialized_tasks)} tasks != {len(current_responses)} responses. '
+                f'Returning empty predictions.'
+            )
+            return []
+
+    def _get_predictions_from_ml_backend(self, serialized_tasks: List[Dict]) -> List[Dict]:
+        result = self.api.make_predictions(serialized_tasks, self.project)
+
+        # response validation
+        if result.is_error:
+            logger.error(f'Error occurred: {result.error_message}')
+            return []
+        elif not isinstance(result.response, dict) or 'results' not in result.response:
+            logger.error(f'ML backend returns an incorrect response, it must be a dict: {result.response}')
+            return []
+        elif not isinstance(result.response['results'], list) or len(result.response['results']) == 0:
+            logger.error(
+                'ML backend returns an incorrect response, results field must be a list with at least one item'
+            )
+            return []
+
+        responses = result.response['results']
+
+        predictions = []
+        if len(serialized_tasks) != len(responses):
+            # Number of tasks and responses are not equal
+            # It can happen if ML backend doesn't support batch processing but only process one task at a time
+            # In the future versions, we may better consider this as an error and deprecate this code branch
+            return self._get_predictions_from_ml_backend_one_by_one(serialized_tasks, responses)
+
+        # ML backend supports batch processing
+        for task, response in zip(serialized_tasks, responses):
+            if isinstance(response, dict):
+                # ML backend can return single prediction per task or multiple predictions
+                response = [response]
+
+            # get all predictions per task
+            for r in response:
+                if 'result' not in r:
+                    logger.error(
+                        f"ML backend returns an incorrect prediction, it should be a dict with the 'result' field:"
+                        f' {r}'
+                    )
+                    continue
+                predictions.append(
+                    {
+                        'task': task['id'],
+                        'result': r['result'],
+                        'score': r.get('score'),
+                        'model_version': r.get('model_version', self.model_version),
+                        'project': task['project'],
+                    }
+                )
+        return predictions
+
     def predict_tasks(self, tasks):
-        self.update_state()
+        model_version = self.update_state()
         if self.not_ready:
             logger.debug(f'ML backend {self} is not ready')
             return
@@ -169,105 +352,18 @@ class MLBackend(models.Model):
 
         # Filter tasks that already contain the current model version in predictions
         tasks = tasks.annotate(predictions_count=Count('predictions')).exclude(
-            Q(predictions_count__gt=0) & Q(predictions__model_version=self.model_version)
+            Q(predictions_count__gt=0) & Q(predictions__model_version=model_version)
         )
         if not tasks.exists():
             logger.debug(f'All tasks already have prediction from model version={self.model_version}')
-            return
+            return model_version
         tasks_ser = TaskSimpleSerializer(tasks, many=True).data
-        ml_api_result = self.api.make_predictions(tasks_ser, self.model_version, self.project)
-        if ml_api_result.is_error:
-            logger.info(f'Prediction not created for project {self}: {ml_api_result.error_message}')
-            return
-
-        if not (isinstance(ml_api_result.response, dict) and 'results' in ml_api_result.response):
-            logger.info(f'ML backend returns an incorrect response, it should be a dict: {ml_api_result.response}')
-            return
-
-        responses = ml_api_result.response['results']
-
-        if len(responses) == 0:
-            logger.warning(f'ML backend returned empty prediction for project {self}')
-            return
-
-        # ML Backend doesn't support batch of tasks, do it one by one
-        elif len(responses) == 1 and len(tasks) != 1:
-            logger.warning(
-                f"'ML backend '{self.title}' doesn't support batch processing of tasks, "
-                f'switched to one-by-one task retrieval'
-            )
-            for task in tasks:
-                self.predict_one_task(task)
-            return
-
-        # wrong result number
-        elif len(responses) != len(tasks_ser):
-            logger.warning(f'ML backend returned response number {len(responses)} != task number {len(tasks_ser)}')
-
-        predictions = []
-        for task, response in zip(tasks_ser, responses):
-            if 'result' not in response:
-                logger.info(
-                    f"ML backend returns an incorrect prediction, it should be a dict with the 'result' field:"
-                    f' {response}'
-                )
-                return
-            predictions.append(
-                {
-                    'task': task['id'],
-                    'result': response['result'],
-                    'score': response.get('score'),
-                    'model_version': response.get('model_version', self.model_version),
-                }
-            )
+        predictions = self._get_predictions_from_ml_backend(tasks_ser)
         with conditional_atomic(predicate=db_is_not_sqlite):
             prediction_ser = PredictionSerializer(data=predictions, many=True)
             prediction_ser.is_valid(raise_exception=True)
-            prediction_ser.save()
-
-    def predict_one_task(self, task, check_state=True):
-        if check_state:
-            self.update_state()
-            if self.not_ready:
-                logger.debug(f'ML backend {self} is not ready to predict {task}')
-                return
-
-        if task.predictions.filter(model_version=self.model_version).exists():
-            # prediction already exists
-            logger.info(
-                f'Skip creating prediction with ML backend {self} for task {task}: model version '
-                f'{self.model_version} is up-to-date'
-            )
-            return
-        ml_api = self.api
-
-        task_ser = TaskSimpleSerializer(task).data
-        ml_api_result = ml_api.make_predictions([task_ser], self.model_version, self.project)
-        if ml_api_result.is_error:
-            logger.info(f'Prediction not created for project {self}: {ml_api_result.error_message}')
-            return
-        results = ml_api_result.response['results']
-        if len(results) == 0:
-            logger.error(f'ML backend returned empty prediction for project {self.id}', extra={'sentry_skip': True})
-            return
-        prediction_response = results[0]
-        task_id = task_ser['id']
-        r = prediction_response['result']
-        score = prediction_response.get('score')
-        with conditional_atomic(predicate=db_is_not_sqlite):
-            prediction = Prediction.objects.create(
-                result=r,
-                score=safe_float(score),
-                model_version=self.model_version,
-                task_id=task_id,
-                project=task.project,
-                cluster=prediction_response.get('cluster'),
-                neighbors=prediction_response.get('neighbors'),
-                mislabeling=safe_float(prediction_response.get('mislabeling', 0)),
-            )
-            logger.debug(f'Prediction {prediction} created')
-
-        return prediction
+            instances = prediction_ser.save()
+        return instances
 
     def interactive_annotating(self, task, context=None, user=None):
         result = {}
@@ -283,7 +379,6 @@ class MLBackend(models.Model):
         ).data
         ml_api_result = self.api.make_predictions(
             tasks=tasks_ser,
-            model_version=self.model_version,
             project=self.project,
             context=context,
         )
@@ -315,14 +410,20 @@ class MLBackend(models.Model):
         return result
 
     @staticmethod
-    def get_versions_(url, project):
-        api = MLApi(url=url)
+    def get_versions_(url, project, auth_method, **kwargs):
+        api = MLApi(url=url, auth_method=auth_method, **kwargs)
         if not isinstance(project, Project):
             project = Project.objects.get(pk=project)
         return api.get_versions(project)
 
     def get_versions(self):
-        return self.get_versions_(self.url, self.project)
+        return self.get_versions_(
+            self.url,
+            self.project,
+            self.auth_method,
+            basic_auth_user=self.basic_auth_user,
+            basic_auth_pass=self.basic_auth_pass,
+        )
 
 
 class MLBackendPredictionJob(models.Model):
@@ -391,6 +492,15 @@ def _validate_ml_api_result(ml_api_result, tasks, curr_logger):
     return True
 
 
+@receiver(pre_delete, sender=MLBackend)
+def modify_project_model_version(sender, instance, **kwargs):
+    project = instance.project
+
+    if project.model_version == instance.title:
+        project.model_version = ''
+        project.save(update_fields=['model_version'])
+
+
 @receiver(post_save, sender=MLBackend)
 def create_ml_webhook(sender, instance, created, **kwargs):
     if not created:
