@@ -539,7 +539,7 @@ class AnnotationsListAPI(GetParentObjectMixin, generics.ListCreateAPIView):
             pass
 
     def perform_create(self, ser):
-        task = self.get_parent_object()
+        task = self.parent_object
         # annotator has write access only to annotations and it can't be checked it after serializer.save()
         user = self.request.user
 
@@ -778,13 +778,7 @@ class PredictionAPI(viewsets.ModelViewSet):
     filterset_fields = ['task', 'task__project', 'project']
 
     def get_queryset(self):
-        if flag_set(
-            'fflag_perf_back_lsdv_4695_update_prediction_query_to_use_direct_project_relation',
-            user='auto',
-        ):
-            return Prediction.objects.filter(project__organization=self.request.user.active_organization)
-        else:
-            return Prediction.objects.filter(task__project__organization=self.request.user.active_organization)
+        return Prediction.objects.filter(project__organization=self.request.user.active_organization)
 
 
 @method_decorator(name='get', decorator=swagger_auto_schema(auto_schema=None))
@@ -804,7 +798,6 @@ class AnnotationConvertAPI(generics.RetrieveAPIView):
     def process_intermediate_state(self, annotation, draft):
         pass
 
-    @swagger_auto_schema(auto_schema=None)
     def post(self, request, *args, **kwargs):
         annotation = self.get_object()
         organization = annotation.project.organization
