@@ -6,8 +6,9 @@ import logging
 import numbers
 import os
 import random
+import traceback
 import uuid
-from typing import Any, Mapping, Optional, cast
+from typing import Any, Mapping, Optional, Union, cast
 from urllib.parse import urljoin
 
 import ujson as json
@@ -27,16 +28,16 @@ from core.utils.params import get_env
 from data_import.models import FileUpload
 from data_manager.managers import PreparedTaskManager, TaskManager
 from django.conf import settings
-from django.contrib.auth.models import AnonymousUser
-from django.core.files.storage import default_storage
 from django.db import OperationalError, models, transaction
-from django.db.models import JSONField, Q
+from django.db.models import CheckConstraint, F, JSONField, Q
+from django.db.models.lookups import GreaterThanOrEqual
 from django.db.models.signals import post_delete, post_save, pre_delete, pre_save
 from django.dispatch import Signal, receiver
 from django.urls import reverse
 from django.utils.timesince import timesince
 from django.utils.timezone import now
 from django.utils.translation import gettext_lazy as _
+from label_studio_sdk.label_interface.objects import PredictionValue
 from rest_framework.exceptions import ValidationError
 from tasks.choices import ActionType
 
@@ -237,6 +238,36 @@ class Task(TaskMixin, models.Model):
         else:
             return []
 
+    def get_lock_exclude_query(self, user):
+        """
+        Get query for excluding annotations from the lock check
+        """
+        SkipQueue = self.project.SkipQueue
+
+        if self.project.skip_queue == SkipQueue.IGNORE_SKIPPED:
+            # IGNORE_SKIPPED: my skipped tasks don't go anywhere
+            # alien's and my skipped annotations are counted as regular annotations
+            q = Q()
+        else:
+            if self.project.skip_queue == SkipQueue.REQUEUE_FOR_ME:
+                # REQUEUE_FOR_ME means: only my skipped tasks go back to me,
+                # alien's skipped annotations are counted as regular annotations
+                q = Q(was_cancelled=True) & Q(completed_by=user)
+            elif self.project.skip_queue == SkipQueue.REQUEUE_FOR_OTHERS:
+                # REQUEUE_FOR_OTHERS: my skipped tasks go to others
+                # alien's skipped annotations are not counted at all
+                q = Q(was_cancelled=True) & ~Q(completed_by=user)
+            else:
+                raise Exception(f'Invalid SkipQueue value: {self.project.skip_queue}')
+
+            # for LSE we also need to exclude rejected queue
+            rejected_q = self.get_rejected_query()
+
+            if rejected_q:
+                q &= rejected_q
+
+        return q | Q(ground_truth=True)
+
     def has_lock(self, user=None):
         """
         Check whether current task has been locked by some user
@@ -245,23 +276,10 @@ class Task(TaskMixin, models.Model):
         """
         from projects.functions.next_task import get_next_task_logging_level
 
-        SkipQueue = self.project.SkipQueue
-
-        if self.project.skip_queue == SkipQueue.REQUEUE_FOR_ME:
-            # REQUEUE_FOR_ME means: only my skipped tasks go back to me,
-            # alien's skipped annotations are counted as regular annotations
-            q = Q(was_cancelled=True) & Q(completed_by=user)
-        elif self.project.skip_queue == SkipQueue.REQUEUE_FOR_OTHERS:
-            # REQUEUE_FOR_OTHERS: my skipped tasks go to others
-            # alien's skipped annotations are not counted at all
-            q = Q(was_cancelled=True) & ~Q(completed_by=user)
-        else:  # SkipQueue.IGNORE_SKIPPED
-            # IGNORE_SKIPPED: my skipped tasks don't go anywhere
-            # alien's and my skipped annotations are counted as regular annotations
-            q = Q()
+        q = self.get_lock_exclude_query(user)
 
         num_locks = self.num_locks_user(user=user)
-        num_annotations = self.annotations.exclude(q | Q(ground_truth=True)).count()
+        num_annotations = self.annotations.exclude(q).count()
         num = num_locks + num_annotations
 
         if num > self.overlap_with_agreement_threshold(num, num_locks):
@@ -339,6 +357,11 @@ class Task(TaskMixin, models.Model):
         num_locks = self.num_locks
         if num_locks < self.overlap:
             lock_ttl = settings.TASK_LOCK_TTL
+            if (
+                flag_set('fflag_feat_all_leap_1534_custom_task_lock_timeout_short', user=user)
+                and self.project.custom_task_lock_ttl
+            ):
+                lock_ttl = self.project.custom_task_lock_ttl
             expire_at = now() + datetime.timedelta(seconds=lock_ttl)
             try:
                 task_lock = TaskLock.objects.get(task=self, user=user)
@@ -427,13 +450,7 @@ class Task(TaskMixin, models.Model):
                     # permission check: resolve uploaded files to the project only
                     file_upload = fast_first(FileUpload.objects.filter(project=project, file=prepared_filename))
                     if file_upload is not None:
-                        if flag_set(
-                            'ff_back_dev_2915_storage_nginx_proxy_26092022_short',
-                            self.project.organization.created_by,
-                        ):
-                            task_data[field] = file_upload.url
-                        else:
-                            task_data[field] = default_storage.url(name=prepared_filename)
+                        task_data[field] = file_upload.url
                     # it's very rare case, e.g. user tried to reimport exported file from another project
                     # or user wrote his django storage path manually
                     else:
@@ -499,17 +516,19 @@ class Task(TaskMixin, models.Model):
     def ensure_unique_groundtruth(self, annotation_id):
         self.annotations.exclude(id=annotation_id).update(ground_truth=False)
 
-    def save(self, *args, **kwargs):
-        if flag_set('ff_back_2070_inner_id_12052022_short', AnonymousUser):
-            if self.inner_id == 0:
-                task = Task.objects.filter(project=self.project).order_by('-inner_id').first()
-                max_inner_id = 1
-                if task:
-                    max_inner_id = task.inner_id
+    def save(self, *args, update_fields=None, **kwargs):
+        if self.inner_id == 0:
+            task = Task.objects.filter(project=self.project).order_by('-inner_id').first()
+            max_inner_id = 1
+            if task:
+                max_inner_id = task.inner_id
 
-                # max_inner_id might be None in the old projects
-                self.inner_id = None if max_inner_id is None else (max_inner_id + 1)
-        super().save(*args, **kwargs)
+            # max_inner_id might be None in the old projects
+            self.inner_id = None if max_inner_id is None else (max_inner_id + 1)
+            if update_fields is not None:
+                update_fields = {'inner_id'}.union(update_fields)
+
+        super().save(*args, update_fields=update_fields, **kwargs)
 
     @staticmethod
     def delete_tasks_without_signals(queryset):
@@ -536,8 +555,8 @@ class Task(TaskMixin, models.Model):
         return result
 
 
-pre_bulk_create = Signal(providing_args=['objs', 'batch_size'])
-post_bulk_create = Signal(providing_args=['objs', 'batch_size'])
+pre_bulk_create = Signal()   # providing args 'objs' and 'batch_size'
+post_bulk_create = Signal()   # providing args 'objs' and 'batch_size'
 
 
 class AnnotationManager(models.Manager):
@@ -665,6 +684,13 @@ class Annotation(AnnotationMixin, models.Model):
         default=None,
         null=True,
     )
+    bulk_created = models.BooleanField(
+        _('bulk created'),
+        default=False,
+        db_default=False,
+        null=True,
+        help_text='Annotation was created in bulk mode',
+    )
 
     class Meta:
         db_table = 'task_completion'
@@ -724,11 +750,20 @@ class Annotation(AnnotationMixin, models.Model):
 
         self.task.save(update_fields=update_fields)
 
-    def save(self, *args, **kwargs):
+    def save(self, *args, update_fields=None, **kwargs):
         request = get_current_request()
         if request:
             self.updated_by = request.user
-        result = super().save(*args, **kwargs)
+            if update_fields is not None:
+                update_fields = {'updated_by'}.union(update_fields)
+
+        unique_list = {result.get('id') for result in (self.result or [])}
+
+        self.result_count = len(unique_list)
+        if update_fields is not None:
+            update_fields = {'result_count'}.union(update_fields)
+        result = super().save(*args, update_fields=update_fields, **kwargs)
+
         self.update_task()
         return result
 
@@ -774,6 +809,7 @@ class TaskLock(models.Model):
         on_delete=models.CASCADE,
         help_text='User who locked this task',
     )
+    created_at = models.DateTimeField(_('created at'), auto_now_add=True, help_text='Creation time', null=True)
 
 
 class AnnotationDraft(models.Model):
@@ -847,7 +883,7 @@ class AnnotationDraft(models.Model):
 
 
 class Prediction(models.Model):
-    """ML backend predictions"""
+    """ML backend / Prompts predictions"""
 
     result = JSONField('result', null=True, default=dict, help_text='Prediction result')
     score = models.FloatField(_('score'), default=None, help_text='Prediction score', null=True)
@@ -897,15 +933,8 @@ class Prediction(models.Model):
         return timesince(self.created_at)
 
     def has_permission(self, user):
-        if flag_set(
-            'fflag_perf_back_lsdv_4695_update_prediction_query_to_use_direct_project_relation',
-            user='auto',
-        ):
-            user.project = self.project  # link for activity log
-            return self.project.has_permission(user)
-        else:
-            user.project = self.task.project  # link for activity log
-            return self.task.project.has_permission(user)
+        user.project = self.project  # link for activity log
+        return self.project.has_permission(user)
 
     @classmethod
     def prepare_prediction_result(cls, result, project):
@@ -965,22 +994,21 @@ class Prediction(models.Model):
 
         self.task.save(update_fields=update_fields)
 
-    def save(self, *args, **kwargs):
+    def save(self, *args, update_fields=None, **kwargs):
         if self.project_id is None and self.task_id:
             logger.warning('project_id is not set for prediction, project_id being set in save method')
             self.project_id = Task.objects.only('project_id').get(pk=self.task_id).project_id
+            if update_fields is not None:
+                update_fields = {'project_id'}.union(update_fields)
 
         # "result" data can come in different forms - normalize them to JSON
-        if flag_set(
-            'fflag_perf_back_lsdv_4695_update_prediction_query_to_use_direct_project_relation',
-            user='auto',
-        ):
-            self.result = self.prepare_prediction_result(self.result, self.project)
-        else:
-            self.result = self.prepare_prediction_result(self.result, self.task.project)
+        self.result = self.prepare_prediction_result(self.result, self.project)
+
+        if update_fields is not None:
+            update_fields = {'result'}.union(update_fields)
         # set updated_at field of task to now()
         self.update_task()
-        return super(Prediction, self).save(*args, **kwargs)
+        return super(Prediction, self).save(*args, update_fields=update_fields, **kwargs)
 
     def delete(self, *args, **kwargs):
         result = super().delete(*args, **kwargs)
@@ -988,10 +1016,196 @@ class Prediction(models.Model):
         self.update_task()
         return result
 
+    @classmethod
+    def create_no_commit(
+        cls, project, label_interface, task_id, data, model_version, model_run
+    ) -> Optional['Prediction']:
+        """
+        Creates a Prediction object from the given result data, without committing it to the database.
+        or returns None if it fails.
+
+        Args:
+            project: The Project object to associate with the Prediction object.
+            label_interface: The LabelInterface object to use to create regions from the result data.
+            data: The data to create the Prediction object with.
+                    Must contain keys that match control tags in labeling configuration
+                    Example:
+                        ```
+                        {
+                            'my_choices': 'positive',
+                            'my_textarea': 'generated document summary'
+                        }
+                        ```
+            task_id: The ID of the Task object to associate with the Prediction object.
+            model_version: The model version that produced the prediction.
+            model_run: The model run that created the prediction.
+        """
+        try:
+
+            # given the data receive, create annotation regions in LS format
+            # e.g. {"sentiment": "positive"} -> {"value": {"choices": ["positive"]}, "from_name": "", "to_name": "", ..}
+            pred = PredictionValue(result=label_interface.create_regions(data)).model_dump()
+
+            prediction = Prediction(
+                project=project,
+                task_id=int(task_id),
+                model_version=model_version,
+                model_run=model_run,
+                score=1.0,  # Setting to 1.0 for now as we don't get back a score
+                result=pred['result'],
+            )
+            return prediction
+        except Exception as exc:
+            # TODO: handle exceptions better
+            logger.error(
+                f'Error creating prediction for task {task_id} with {data}: {exc}. '
+                f'Traceback: {traceback.format_exc()}'
+            )
+
     class Meta:
         db_table = 'prediction'
 
 
+class FailedPrediction(models.Model):
+    """
+    Class for storing failed prediction(s) for a task
+    """
+
+    message = models.TextField(
+        _('message'),
+        default=None,
+        blank=True,
+        null=True,
+        help_text='The message explaining why generating this prediction failed',
+    )
+    error_type = models.CharField(
+        _('error_type'),
+        max_length=512,
+        default=None,
+        null=True,
+        help_text='The type of error that caused prediction to fail',
+    )
+    ml_backend_model = models.ForeignKey(
+        'ml.MLBackend',
+        on_delete=models.SET_NULL,
+        related_name='failed_predictions',
+        null=True,
+        help_text='An ML Backend instance that created the failed prediction.',
+    )
+    model_version = models.TextField(
+        _('model version'),
+        default=None,
+        blank=True,
+        null=True,
+        help_text='A string value that for model version that produced the failed prediction. Used in both live models and when uploading offline predictions.',
+    )
+    model_run = models.ForeignKey(
+        'ml_models.ModelRun',
+        on_delete=models.CASCADE,
+        related_name='failed_predictions',
+        null=True,
+        help_text='A run of a ModelVersion that created the failed prediction.',
+    )
+    project = models.ForeignKey(
+        'projects.Project', on_delete=models.CASCADE, related_name='failed_predictions', null=True
+    )
+    task = models.ForeignKey('tasks.Task', on_delete=models.CASCADE, related_name='failed_predictions')
+    created_at = models.DateTimeField(_('created at'), auto_now_add=True)
+
+
+class PredictionMeta(models.Model):
+    prediction = models.OneToOneField(
+        'Prediction',
+        on_delete=models.CASCADE,
+        null=True,
+        blank=True,
+        related_name='meta',
+        help_text=_('Reference to the associated prediction'),
+    )
+    failed_prediction = models.OneToOneField(
+        'FailedPrediction',
+        on_delete=models.CASCADE,
+        null=True,
+        blank=True,
+        related_name='meta',
+        help_text=_('Reference to the associated failed prediction'),
+    )
+    inference_time = models.FloatField(
+        _('inference time'), null=True, blank=True, help_text=_('Time taken for inference in seconds')
+    )
+    prompt_tokens_count = models.IntegerField(
+        _('prompt tokens count'), null=True, blank=True, help_text=_('Number of tokens in the prompt')
+    )
+    completion_tokens_count = models.IntegerField(
+        _('completion tokens count'), null=True, blank=True, help_text=_('Number of tokens in the completion')
+    )
+    total_tokens_count = models.IntegerField(
+        _('total tokens count'), null=True, blank=True, help_text=_('Total number of tokens')
+    )
+    prompt_cost = models.FloatField(_('prompt cost'), null=True, blank=True, help_text=_('Cost of the prompt'))
+    completion_cost = models.FloatField(
+        _('completion cost'), null=True, blank=True, help_text=_('Cost of the completion')
+    )
+    total_cost = models.FloatField(_('total cost'), null=True, blank=True, help_text=_('Total cost'))
+    extra = models.JSONField(_('extra'), null=True, blank=True, help_text=_('Additional metadata in JSON format'))
+
+    @classmethod
+    def create_no_commit(cls, data, prediction: Union[Prediction, FailedPrediction]) -> Optional['PredictionMeta']:
+        """
+        Creates a PredictionMeta object from the given result data, without committing it to the database.
+        or returns None if it fails.
+
+        Args:
+            data: The data to create the PredictionMeta object with.
+                  Example:
+                    {
+                        'total_cost_usd': 0.1,
+                        'prompt_cost_usd': 0.05,
+                        'completion_cost_usd': 0.05,
+                        'prompt_tokens': 10,
+                        'completion_tokens': 10,
+                        'inference_time': 0.1
+                    }
+            prediction: The Prediction or FailedPrediction object to associate with the PredictionMeta object.
+
+        Returns:
+            The PredictionMeta object created from the given data, or None if it fails.
+        """
+        try:
+            prediction_meta = PredictionMeta(
+                total_cost=data.get('total_cost_usd'),
+                prompt_cost=data.get('prompt_cost_usd'),
+                completion_cost=data.get('completion_cost_usd'),
+                prompt_tokens_count=data.get('prompt_tokens'),
+                completion_tokens_count=data.get('completion_tokens'),
+                total_tokens_count=data.get('prompt_tokens', 0) + data.get('completion_tokens', 0),
+                inference_time=data.get('inference_time'),
+            )
+            if isinstance(prediction, Prediction):
+                prediction_meta.prediction = prediction
+            else:
+                prediction_meta.failed_prediction = prediction
+            logger.debug(f'Created prediction meta: {prediction_meta}')
+            return prediction_meta
+        except Exception as exc:
+            logger.error(f'Error creating prediction meta with {data}: {exc}. Traceback: {traceback.format_exc()}')
+
+    class Meta:
+        db_table = 'prediction_meta'
+        verbose_name = _('Prediction Meta')
+        verbose_name_plural = _('Prediction Metas')
+        constraints = [
+            CheckConstraint(
+                # either prediction or failed_prediction should be not null
+                check=(
+                    (Q(prediction__isnull=False) & Q(failed_prediction__isnull=True))
+                    | (Q(prediction__isnull=True) & Q(failed_prediction__isnull=False))
+                ),
+                name='prediction_or_failed_prediction_not_null',
+            )
+        ]
+
+
 @receiver(post_delete, sender=Task)
 def update_all_task_states_after_deleting_task(sender, instance, **kwargs):
     """after deleting_task
@@ -1179,10 +1393,15 @@ def bulk_update_stats_project_tasks(tasks, project=None):
         maximum_annotations = project.maximum_annotations
         # update filters if we can use overlap
         if use_overlap:
-            # finished tasks
-            finished_tasks = tasks.filter(
-                Q(total_annotations__gte=maximum_annotations) | Q(total_annotations__gte=1, overlap=1)
+            # following definition of `completed_annotations` above, count cancelled annotations
+            # as completed if project is in IGNORE_SKIPPED mode
+            completed_annotations_f_expr = F('total_annotations')
+            if project.skip_queue == project.SkipQueue.IGNORE_SKIPPED:
+                completed_annotations_f_expr += F('cancelled_annotations')
+            finished_q = Q(GreaterThanOrEqual(completed_annotations_f_expr, maximum_annotations)) | Q(
+                GreaterThanOrEqual(completed_annotations_f_expr, 1), overlap=1
             )
+            finished_tasks = tasks.filter(finished_q)
             finished_tasks_ids = finished_tasks.values_list('id', flat=True)
             tasks.update(is_labeled=Q(id__in=finished_tasks_ids))
 
