@@ -34,6 +34,8 @@ except ImportError:
     print('\n\n !!! Please, pip install pytest-env \n\n')
     exit(-100)
 
+from label_studio.tests.sdk.fixtures import *  # noqa: F403
+
 from .utils import (
     azure_client_mock,
     create_business,
@@ -145,7 +147,9 @@ def s3_with_hypertext_s3_links(s3):
     s3.put_object(
         Bucket=bucket_name,
         Key='test.json',
-        Body=json.dumps({'text': '<a href="s3://hypertext-bucket/file with /spaces and\' / \' / quotes.jpg"/>'}),
+        Body=json.dumps(
+            {'text': '<a href="s3://pytest-s3-jsons-hypertext/file with /spaces and\' / \' / quotes.jpg"/>'}
+        ),
     )
     yield s3
 
@@ -157,7 +161,11 @@ def s3_with_partially_encoded_s3_links(s3):
     s3.put_object(
         Bucket=bucket_name,
         Key='test.json',
-        Body=json.dumps({'text': '<a href="s3://hypertext-bucket/file with /spaces and\' / \' / %2Bquotes%3D.jpg"/>'}),
+        Body=json.dumps(
+            {
+                'text': '<a href="s3://pytest-s3-json-partially-encoded/file with /spaces and\' / \' / %2Bquotes%3D.jpg"/>'
+            }
+        ),
     )
     yield s3
 
