@@ -300,7 +300,7 @@ def build_urls(project_id, task_id, annotation_id):
                 if 'pk' in key:
                     kwargs[key] = 1000  # for example user_pk or project_pk will be 1000
 
-                if key in ['pk', 'step_pk', 'job_id', 'queue_index']:
+                if key in ['pk', 'step_pk', 'job_id', 'queue_index', 'scheduler_index']:
                     kwargs[key] = 1000
                 elif key in ['token', 'uidb64']:
                     kwargs[key] = 1000
